@@ -17,7 +17,6 @@
 from lxml import etree
 from oslo_log import log as logging
 from oslo_utils import units
-import six
 
 try:
     import libvirt
@@ -38,6 +37,7 @@ class LibvirtInspector(virt_inspector.Inspector):
         super(LibvirtInspector, self).__init__(conf)
         # NOTE(sileht): create a connection on startup
         self.connection
+        self.cache = {}
 
     @property
     def connection(self):
@@ -59,7 +59,7 @@ class LibvirtInspector(virt_inspector.Inspector):
                                  'ex': ex}
             raise virt_inspector.InstanceNotFoundException(msg)
         except Exception as ex:
-            raise virt_inspector.InspectorException(six.text_type(ex))
+            raise virt_inspector.InspectorException(str(ex))
 
     def _get_domain_not_shut_off_or_raise(self, instance):
         instance_name = util.instance_name(instance)
@@ -114,7 +114,36 @@ class LibvirtInspector(virt_inspector.Inspector):
             params['interfaceid'] = interfaceid
             params['bridge'] = bridge
 
-            dom_stats = domain.interfaceStats(name)
+            try:
+                dom_stats = domain.interfaceStats(name)
+            except libvirt.libvirtError as ex:
+                LOG.warning(_("Error from libvirt when running instanceStats, "
+                              "This may not be harmful, but please check : "
+                              "%(ex)s") % {'ex': ex})
+                continue
+
+            # Retrieve previous values
+            prev = self.cache.get(name)
+
+            # Store values for next call
+            self.cache[name] = dom_stats
+
+            if prev:
+                # Compute stats
+                rx_delta = dom_stats[0] - prev[0]
+                tx_delta = dom_stats[4] - prev[4]
+
+                # Avoid negative values
+                if rx_delta < 0:
+                    rx_delta = dom_stats[0]
+                if tx_delta < 0:
+                    tx_delta = dom_stats[4]
+            else:
+                LOG.debug('No delta meter predecessor for %s / %s' %
+                          (instance.id, name))
+                rx_delta = 0
+                tx_delta = 0
+
             yield virt_inspector.InterfaceStats(name=name,
                                                 mac=mac_address,
                                                 fref=fref,
@@ -123,10 +152,12 @@ class LibvirtInspector(virt_inspector.Inspector):
                                                 rx_packets=dom_stats[1],
                                                 rx_errors=dom_stats[2],
                                                 rx_drop=dom_stats[3],
+                                                rx_bytes_delta=rx_delta,
                                                 tx_bytes=dom_stats[4],
                                                 tx_packets=dom_stats[5],
                                                 tx_errors=dom_stats[6],
-                                                tx_drop=dom_stats[7])
+                                                tx_drop=dom_stats[7],
+                                                tx_bytes_delta=tx_delta)
 
     @staticmethod
     def _get_disk_devices(domain):
@@ -202,7 +233,7 @@ class LibvirtInspector(virt_inspector.Inspector):
         # have holes according to
         # https://libvirt.org/git/?p=libvirt.git;a=blob;f=src/libvirt-domain.c
         # virConnectGetAllDomainStats()
-        for vcpu in six.moves.range(stats.get('vcpu.maximum', 0)):
+        for vcpu in range(stats.get('vcpu.maximum', 0)):
             try:
                 cpu_time += (stats.get('vcpu.%s.time' % vcpu) +
                              stats.get('vcpu.%s.wait' % vcpu))
