@@ -18,6 +18,8 @@
 """Tests for ceilometer agent manager"""
 import copy
 import datetime
+import shutil
+import tempfile
 from unittest import mock
 
 import fixtures
@@ -102,7 +104,7 @@ class TestManager(base.BaseTestCase):
         mgr = manager.AgentManager(0, self.conf,
                                    namespaces=['ipmi'])
         # 8 pollsters for Node Manager
-        self.assertEqual(12, len(mgr.extensions))
+        self.assertEqual(13, len(mgr.extensions))
 
     # Skip loading pollster upon ExtensionLoadError
     @mock.patch('ceilometer.ipmi.pollsters.node._Base.__init__',
@@ -378,6 +380,17 @@ class TestPollingAgent(BaseAgent):
         super(TestPollingAgent, self).setUp()
         self.mgr = self.create_manager()
         self.mgr.extensions = self.create_extension_list()
+        ks_client = mock.Mock(auth_token='fake_token')
+        ks_client.projects.get.return_value = mock.Mock(
+            name='admin', id='4465ecd1438b4d23a866cf8447387a7b'
+        )
+        ks_client.users.get.return_value = mock.Mock(
+            name='admin', id='c0c935468e654d5a8baae1a08adf4dfb'
+        )
+        self.useFixture(fixtures.MockPatch(
+            'ceilometer.keystone_client.get_client',
+            return_value=ks_client))
+        self.ks_client = ks_client
         self.setup_polling()
 
     @mock.patch('ceilometer.polling.manager.PollingManager')
@@ -422,6 +435,76 @@ class TestPollingAgent(BaseAgent):
         self.assertIn(60, polling_tasks.keys())
         self.assertNotIn(10, polling_tasks.keys())
 
+    @mock.patch('glob.glob')
+    @mock.patch('ceilometer.declarative.load_definitions')
+    def test_setup_polling_dynamic_pollster_namespace(self, load_mock,
+                                                      glob_mock):
+        glob_mock.return_value = ['test.yml']
+        load_mock.return_value = [{
+            'name': "test.dynamic.pollster",
+            'namespaces': "dynamic",
+            'sample_type': 'gauge',
+            'unit': 'test',
+            'endpoint_type': 'test',
+            'url_path': 'test',
+            'value_attribute': 'test'
+        }, {
+            'name': "test.compute.central.pollster",
+            'sample_type': 'gauge',
+            'namespaces': ["compute", "central"],
+            'unit': 'test',
+            'endpoint_type': 'test',
+            'url_path': 'test',
+            'value_attribute': 'test'
+        }, {
+            'name': "test.compute.pollster",
+            'namespaces': ["compute"],
+            'sample_type': 'gauge',
+            'unit': 'test',
+            'endpoint_type': 'test',
+            'url_path': 'test',
+            'value_attribute': 'test'
+        }, {
+            'name': "test.central.pollster",
+            'sample_type': 'gauge',
+            'unit': 'test',
+            'endpoint_type': 'test',
+            'url_path': 'test',
+            'value_attribute': 'test'
+        }]
+        mgr = manager.AgentManager(0, self.CONF, namespaces=['dynamic'])
+        self.assertEqual(len(mgr.extensions), 1)
+        self.assertEqual(
+            mgr.extensions[0].definitions.configurations['name'],
+            'test.dynamic.pollster')
+
+        mgr = manager.AgentManager(0, self.CONF)
+        self.assertEqual(
+            mgr.extensions[-3].definitions.configurations['name'],
+            'test.compute.central.pollster')
+        self.assertEqual(
+            mgr.extensions[-2].definitions.configurations['name'],
+            'test.compute.pollster')
+        self.assertEqual(
+            mgr.extensions[-1].definitions.configurations['name'],
+            'test.central.pollster')
+
+        mgr = manager.AgentManager(0, self.CONF, namespaces=['compute'])
+        self.assertEqual(
+            mgr.extensions[-2].definitions.configurations['name'],
+            'test.compute.central.pollster')
+        self.assertEqual(
+            mgr.extensions[-1].definitions.configurations['name'],
+            'test.compute.pollster')
+
+        mgr = manager.AgentManager(0, self.CONF, ['central'])
+        self.assertEqual(
+            mgr.extensions[-2].definitions.configurations['name'],
+            'test.compute.central.pollster')
+        self.assertEqual(
+            mgr.extensions[-1].definitions.configurations['name'],
+            'test.central.pollster')
+
     def test_setup_polling_task_same_interval(self):
         self.polling_cfg['sources'].append({
             'name': 'test_polling_1',
@@ -613,9 +696,13 @@ class TestPollingAgent(BaseAgent):
         self.setup_polling()
         polling_task = list(self.mgr.setup_polling_tasks().values())[0]
         polling_task.poll_and_notify()
-        LOG.info.assert_called_with(
-            'Polling pollster %(poll)s in the context of %(src)s',
-            {'poll': 'test', 'src': 'test_polling'})
+
+        LOG.info.assert_has_calls([
+            mock.call('Polling pollster %(poll)s in the context of %(src)s',
+                      {'poll': 'test', 'src': 'test_polling'}),
+            mock.call('Finished polling pollster %(poll)s in the context '
+                      'of %(src)s', {'poll': 'test', 'src': 'test_polling'})
+        ])
 
     @mock.patch('ceilometer.polling.manager.LOG')
     def test_skip_polling_and_notify_with_no_resources(self, LOG):
@@ -784,7 +871,10 @@ class TestPollingAgentPartitioned(BaseAgent):
 
     def setUp(self):
         super(TestPollingAgentPartitioned, self).setUp()
-        self.CONF.set_override("backend_url", "zake://", "coordination")
+        self.tempdir = tempfile.mkdtemp()
+        self.CONF.set_override("backend_url", "file://%s" % self.tempdir,
+                               "coordination")
+        self.addCleanup(shutil.rmtree, self.tempdir, ignore_errors=True)
         self.hashring = mock.MagicMock()
         self.hashring.belongs_to_self = mock.MagicMock()
         self.hashring.belongs_to_self.return_value = True
